#!/usr/bin/python

import sys
import os
from formencode import PostMultipart
import getopt
import httplib
from urlparse import urlparse
import Image
import gzip

usage="""
Usage : shuttup-cli [args] <url> <album> <files>
        shuttup-cli [-l|--list-albums] <url>

   CLI Client for batch uploading photos to a Shutterbug server.

   'url' is the base url of a shutterbug server, i.e: 
       http://mysite.com/shutterbug
   'album' is the string name of an album.  If the album doesn't exist, 
       it will be created.
   'files' is one or more file names to upload to the server.

   args is one or more of:
        -u --username <name> :  Specify admin user login name
        -p --password <pass> :  Specify admin user password
        -l --list-albums     :  List available albums
        -t --title <title>   :  Specify a default photo title.
        -d --desc <desc>     :  Specify a default photo description.
        -r --resize <WxH>    :  Resize photos to WIDTHxHEIGHT
	-A --album-name      :  Name for new album
        -D --album-desc      :  Description for new album
"""

class ShutterbugCLI:
	def __init__(self):
		self.url = ''
		self.album = 0 
		self.files = []
		self.username = ''
		self.password = ''
		self.title = ''
		self.desc = ''
		self.width = 0
		self.height = 0
		self.listalbums = False
		self.albumname = 'New Album'
		self.albumdesc = 'Album created by Shuttup'
		self.gthumb = False

	def parse_options(self, argv):
		short_arg = 'u:p:lt:d:r:hA:D:g'
		long_arg = ['username=','password=','list-albums',\
			'title=','desc=','resize=','help','album-name',\
			'ambum-desc']
		try:
			opts,args = getopt.gnu_getopt(argv, short_arg, long_arg)
		except:
			print usage
			sys.exit(2)

		for o, a in opts:
			if o in ('-h','--help'):
				print usage
				sys.exit(2)
			if o in ('-u','--username'):
				self.username = a
			if o in ('-p','--password'):
				self.password = a
			if o in ('-l','--list-albums'):
				self.listalbums = True
			if o in ('-t','--title'):
				self.title = a
			if o in ('-d','--description'):
				self.desc = a
			if o in ('-g','--gthumb'):
				self.gthumb = True
			if o in ('-r','--resize'):
				try:
					width, height = a.split('x')
					self.width = int(width)
					self.height = int(height)
				except:
					print 'Error parsing resize argument '\
							+ a
					print usage
					sys.exit(2)
			if o in ('-A','--album-name'):
				self.albumname = a
			if o in ('-D','--album-desc'):
				self.albumdesc = a

		if not len(args):
			print 'Too few arguments specified'
			print usage
			sys.exit(2)

		if self.listalbums:
			if len(args) > 1:
				print 'Too many args for listing albums'
				print usage
				sys.exit(2)
				
		else:
			if len(args) < 3:
				print 'Too few arguments, must be at least 3'
				print usage
				sys.exit(2)
		
		try:
			self.url = urlparse(args[0])
		except:
			print 'Not a valid url ' + args[0]
			self.exit(2)

		if not self.listalbums:
			self.album = int(args[1])
			self.files = args[2:]

		for file in self.files:
			if not os.path.exists(file):
				print 'No such file ' + file
				print usage
				sys.exit(2)

	def _orient(self, img):
		exif = img._getexif()
		o = None
	        if exif and exif.has_key(274):
			o = exif[274]
		
		if o:
			if o == 2 and img.size[0] < img.size[1]:
				img = img.transpose(IMG.FLIP_LEFT_RIGHT)
                        if o == 3 and img.size[0] < img.size[1]:
                                img = img.transpose(IMG.ROTATE_180)
                        if o == 4 and img.size[0] < img.size[1]:
                                img = img.transpose(IMG.FLIP_TOP_BOTTOM)
                        if o == 5 and img.size[0] < img.size[1]:
                                img = img.transpose(IMG.ROTATE_270)
                                img = img.transpose(IMG.FLIP_LEFT_RIGHT)
                        if o == 6 and img.size[0] > img.size[1]:
                                img = img.transpose(IMG.ROTATE_270)
                        if o == 7 and img.size[0] > img.size[1]:
                                img = img.transpose(IMG.ROTATE_90)
                                img = img.transpose(IMG.FLIP_LEFT_RIGHT)
                        if o == 8 and img.size[0] > img.size[1]:
                                img = img.transpose(IMG.ROTATE_90)      
		return img


	def list_albums(self):
		conn = httplib.HTTPConnection(self.url.netloc)
		conn.request("GET",self.url.path.strip('/') + "/albums")
		resp = conn.getresponse()
		albums = []
		if resp.status == 200:
			for line in resp.read().splitlines():
				name = line.split(',', 1)[1]
				id = line.split(',', 1)[0]
				albums.append((id, name))
			return albums	
		else:
			print 'Could not retrieve albums list : %d %s' % \
				(resp.status, resp.reason)
			print 'URL was %s' % (self.url.geturl().strip('/') + \
					'/albums')
			return []
					
	def upload_images(self):
		photoposter = PostMultipart(\
			self.url.geturl().strip('/') + '/image/save',
			login_url=self.url.geturl().strip('/') + '/login',
			username=self.username, password=self.password)
			
		if not self.album:
			sys.stdout.write('Creating album...')
			sys.stdout.flush()
			# Create the new album if requested.
			albumposter = PostMultipart(\
				self.url.geturl().strip('/') + '/album/save',
				login_url=self.url.geturl().strip('/')+'/login',
				username=self.username, password=self.password)

			fields=[('name',self.albumname),\
				('desc',self.albumdesc),\
				('id','new')]
			result = albumposter.post_multipart(fields, [])
			self.album = int(self.list_albums()[-1][0])
			sys.stdout.write('done.\n')
			sys.stdout.flush()

		for file in self.files:
			print 'Processing file : %s' % file
			sys.stdout.write('Reading...')
			sys.stdout.flush()
			from cStringIO import StringIO
			img = None
			if self.width:
				img = Image.open(file)
				img = self._orient(img)

				sys.stdout.write('Resizing (%d x %d)...' % \
						(self.width, self.height))
				sys.stdout.flush()
				if img.size[0] > img.size[1]:
					img = img.resize((self.width, \
							self.height), \
							Image.ANTIALIAS)
				else:
					img = img.resize((self.height, \
							self.width), \
							Image.ANTIALIAS)

			sys.stdout.write('Uploading...')
			sys.stdout.flush()
			if self.width:
				value = img.tostring('jpeg', 'RGB')
			else:
				fp = open(file, "rb")
				value = fp.read()
				fp.close()

			# special case for gThumb import.  if gThumb is true,
			# then try and read the comments and titles from 
			# any comment files that may exist
			if self.gthumb:
				desc = ''
				commfile = os.path.join(os.path.dirname(file),
						'.comments',os.path.basename(file) + '.xml')
				if os.path.isfile(commfile):
				        gzipfile = gzip.GzipFile(commfile, \
									"rb", 3)
					content = gzipfile.read()
					start = content.find("<Note>")
					end = content.find("</Note>", start)
					desc = content[start+6:end]
				else:
					desc = self.desc
			else:
				desc = self.desc

			fields=[('name',self.title),('desc',desc),\
				('album',str(self.album))]
			files = [('image',file,value)] 
			photoposter.post_multipart(fields, files)
			sys.stdout.write('done.\n')
			sys.stdout.flush()

		return

if __name__=='__main__':
	cli = ShutterbugCLI()
	cli.parse_options(sys.argv[1:])
	if cli.listalbums:
		albums = cli.list_albums()
		print 'ID       Name'
		for a in albums:
			print '%-8d %s' % (int(a[0]), a[1])
	else:
		cli.upload_images()
